==> pyutilib.workflow-1.0.tar.gz: pyutilib.workflow-1.0/pyutilib/workflow/tests/test_core.py <==

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+"..")
currdir = dirname(abspath(__file__))+os.sep

import pyutilib.th as unittest
import pyutilib.workflow


class TaskA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x','y')
        self.outputs.declare('z')

    def execute(self):
        self.z = self.x + self.y


class TaskB(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('a')
        self.inputs.declare('i')
        self.outputs.declare('b')

    def execute(self):
        self.b = 100*self.i+2*self.a


class TaskBB(TaskB):

    def __init__(self, *args, **kwds):
        TaskB.__init__(self, *args, **kwds)

    def _create_parser(self):
        pyutilib.workflow.Task._create_parser(self)
        self.parser.add_option("--a",dest="a", type='int')
        self.parser.add_option("--i",dest="i", type='int')


class TaskC(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('i')
        self.outputs.declare('o')

    def execute(self):
        self.o = 10*self.i


class TaskCC(TaskC):

    def __init__(self, *args, **kwds):
        TaskC.__init__(self, *args, **kwds)

    def _create_parser(self):
        pyutilib.workflow.Task._create_parser(self)
        self.parser.add_option("--i",dest="i", type='int')


class TaskD(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('i')
        self.outputs.declare('o')
        self.outputs.declare('z')

    def execute(self):
        self.o = 10*self.i
        self.z = -999


class TaskE(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('I')
        self.outputs.declare('O')

    def execute(self):
        self.O = self.I


class Test(unittest.TestCase):

    def test1(self):
        pyutilib.workflow.reset_task_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        C = TaskC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A)
        w.add(A)  # This is redundant, but it double-checks the logic for adding tasks
        #
        # Print workflow
        #
        ##print "..."
        self.failUnlessEqual(str(w),\
"""Workflow:
Task4 prev: [] next: [2, 3] resources: []
Task2 prev: [4] next: [1] resources: []
Task3 prev: [4] next: [1] resources: []
Task1 prev: [2, 3] next: [5] resources: []
Task5 prev: [1] next: [] resources: []""")
        self.failUnlessEqual(w(i=3, a=2), {'z':334})

    def test1c(self):
        pyutilib.workflow.reset_task_counter()
        # Define tasks
        A = TaskA()
        E1 = TaskE()
        E2 = TaskE()
        C = TaskC()
        # Establish connections
        A.inputs.x = E1.outputs.O
        A.inputs.y = E2.outputs.O
        E1.inputs.I = C.outputs.o
        E2.inputs.I = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A)
        #
        # Print workflow
        #
        ##print "..."
        self.failUnlessEqual(str(w),\
"""Workflow:
Task5 prev: [] next: [4] resources: []
Task4 prev: [5] next: [2, 3] resources: []
Task2 prev: [4] next: [1] resources: []
Task3 prev: [4] next: [1] resources: []
Task1 prev: [2, 3] next: [6] resources: []
Task6 prev: [1] next: [] resources: []""")
        self.failUnlessEqual(w(i=3), {'z':60})

    def test1b(self):
        pyutilib.workflow.reset_task_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        C = TaskC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A,loadall=False)
        self.failUnlessEqual(str(w),"Workflow:\nTask4 prev: [] next: [] resources: []")

    def test1a(self):
        pyutilib.workflow.reset_task_counter()
        # Create workflow
        w = pyutilib.workflow.Workflow()
        self.failUnlessEqual(str(w),"Workflow:\nTask1 prev: [] next: [] resources: []")
        w.add(pyutilib.workflow.task.NoTask)
        self.failUnlessEqual(str(w),"Workflow:\nTask1 prev: [] next: [] resources: []")

    def test2(self):
        pyutilib.workflow.reset_task_counter()
        A = TaskA(name="A")
        B = TaskB()
        A.inputs.x = B.outputs.b
        self.failUnlessEqual(repr(A), "A prev: [2] next: [] resources: []")

    def test3(self):
        pyutilib.workflow.reset_task_counter()
        A = TaskA()
        B = TaskB()
        A.inputs.x = B.outputs.b
        self.failUnlessEqual(str(A.inputs), """Task1-inputs:
    x = DirectConnection: from=(Task2 prev: [] next: [1] resources: []) to=(set([Task1 prev: [2] next: [] resources: []]))
    y = UnknownConnection: from=(Task0 prev: [] next: [] resources: []) to=(set([Task1 prev: [2] next: [] resources: []]))
Mode:  inputs
Owner: Task1 prev: [2] next: [] resources: []""")
        self.failUnlessEqual(str(A.outputs),  'Task1-outputs:\n    z = DirectConnection: from=(Task1 prev: [2] next: [] resources: []) to=(set([]))\nMode:  outputs\nOwner: Task1 prev: [2] next: [] resources: []')
        self.failUnlessEqual(repr(A.inputs),"""InputConnections(x = DirectConnection: from=(Task2 prev: [] next: [1] resources: []) to=(set([Task1 prev: [2] next: [] resources: []])), y = UnknownConnection: from=(Task0 prev: [] next: [] resources: []) to=(set([Task1 prev: [2] next: [] resources: []])))""")

    def test4(self):
        pyutilib.workflow.reset_task_counter()
        A = TaskA()
        A.inputs['x'] = 2
        self.failUnlessEqual(A.inputs['x'].get_value(), 2)
        self.failUnlessEqual(repr(A), "Task1 prev: [] next: [] resources: []")

    def test5(self):
        pyutilib.workflow.reset_task_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        C = TaskC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A)
        w.set_options(['--i=3','--a=2'])
        try:
            w()
            self.fail("Expected TypeError because the inputs are not defined")
        except TypeError:
            pass

    def test5a(self):
        pyutilib.workflow.reset_task_counter()
        # Define tasks
        A = TaskA()
        B = TaskBB()
        C = TaskCC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A)
        w.set_options(['--i=3','--a=2'])
        self.failUnlessEqual(w(), {'z':334})
        # This doesn't work, since we can't merge options that aren't
        # in groups.  The soln is to move to argparse, but I'll save that for later
        #w.print_help()

    def test_error1(self):
        pyutilib.workflow.reset_task_counter()
        A = TaskA()
        try:
            A.inputs['inputs'] = 2
            self.fail("Expected ValueError because we're using the reserved 'inputs' attribute name")
        except ValueError:
            pass

    def test_error2(self):
        pyutilib.workflow.reset_task_counter()
        A = TaskA()
        try:
            print A.inputs._foo
            self.fail("Should have generated error for unknown attribute _foo")
        except AttributeError:
            pass

    def test_error3(self):
        pyutilib.workflow.reset_task_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        D = TaskD()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = D.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        try:
            w.add(A)
            self.fail("Expected ValueError because there are multiple outputs with the same name")
        except ValueError:
            pass

    def test_error4(self):
        pyutilib.workflow.reset_task_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        C = TaskC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        try:
            w(i=3, xx=2)
            self.fail("Expected ValueError because input 'xx' is unknown")
        except ValueError:
            pass

if __name__ == "__main__":
    unittest.main()

==> pyutilib.workflow-1.1.tar.gz: pyutilib.workflow-1.1/pyutilib/workflow/tests/test_core.py <==

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+"..")
currdir = dirname(abspath(__file__))+os.sep

import pyutilib.th as unittest
import pyutilib.workflow


class TaskA(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('x')
        self.inputs.declare('y')
        self.outputs.declare('z')

    def execute(self):
        self.z = self.x + self.y


class TaskB(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('a')
        self.inputs.declare('i')
        self.outputs.declare('b')

    def execute(self):
        self.b = 100*self.i+2*self.a


class TaskBB(TaskB):

    def __init__(self, *args, **kwds):
        TaskB.__init__(self, *args, **kwds)

    def _create_parser(self):
        pyutilib.workflow.Task._create_parser(self)
        self.parser.add_option("--a",dest="a", type='int')
        self.parser.add_option("--i",dest="i", type='int')


class TaskC(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('i')
        self.outputs.declare('o')

    def execute(self):
        self.o = 10*self.i


class TaskCC(TaskC):

    def __init__(self, *args, **kwds):
        TaskC.__init__(self, *args, **kwds)

    def _create_parser(self):
        pyutilib.workflow.Task._create_parser(self)
        self.parser.add_option("--i",dest="i", type='int')


class TaskD(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('i')
        self.outputs.declare('o')
        self.outputs.declare('z')

    def execute(self):
        self.o = 10*self.i
        self.z = -999


class TaskE(pyutilib.workflow.Task):

    def __init__(self, *args, **kwds):
        pyutilib.workflow.Task.__init__(self, *args, **kwds)
        self.inputs.declare('I')
        self.outputs.declare('O')

    def execute(self):
        self.O = self.I


class Test(unittest.TestCase):

    def test1(self):
        pyutilib.workflow.reset_id_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        C = TaskC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A)
        w.add(A)  # This is redundant, but it double-checks the logic for adding tasks
        #
        # Print workflow
        #
        ##print "..."
        self.failUnlessEqual(str(w),\
"""Workflow:
Task4 prev: [] next: [2, 3] resources: []
Task2 prev: [4] next: [1] resources: []
Task3 prev: [4] next: [1] resources: []
Task1 prev: [2, 3] next: [5] resources: []
Task5 prev: [1] next: [] resources: []""")
        self.failUnlessEqual(w(i=3, a=2), {'z':334})

    def test1c(self):
        pyutilib.workflow.reset_id_counter()
        # Define tasks
        A = TaskA()
        E1 = TaskE()
        E2 = TaskE()
        C = TaskC()
        # Establish connections
        A.inputs.x = E1.outputs.O
        A.inputs.y = E2.outputs.O
        E1.inputs.I = C.outputs.o
        E2.inputs.I = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A)
        #
        # Print workflow
        #
        ##print "..."
        self.failUnlessEqual(str(w),\
"""Workflow:
Task5 prev: [] next: [4] resources: []
Task4 prev: [5] next: [2, 3] resources: []
Task2 prev: [4] next: [1] resources: []
Task3 prev: [4] next: [1] resources: []
Task1 prev: [2, 3] next: [6] resources: []
Task6 prev: [1] next: [] resources: []""")
        self.failUnlessEqual(w(i=3), {'z':60})

    def test1b(self):
        pyutilib.workflow.reset_id_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        C = TaskC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A,loadall=False)
        self.failUnlessEqual(str(w),"Workflow:\nTask4 prev: [] next: [] resources: []")

    def test1a(self):
        pyutilib.workflow.reset_id_counter()
        # Create workflow
        w = pyutilib.workflow.Workflow()
        self.failUnlessEqual(str(w),"Workflow:\nTask1 prev: [] next: [] resources: []")
        w.add(pyutilib.workflow.task.NoTask)
        self.failUnlessEqual(str(w),"Workflow:\nTask1 prev: [] next: [] resources: []")

    def test2(self):
        pyutilib.workflow.reset_id_counter()
        A = TaskA(name="A")
        B = TaskB()
        A.inputs.x = B.outputs.b
        self.failUnlessEqual(repr(A), "A prev: [2] next: [] resources: []")

    def test3(self):
        pyutilib.workflow.reset_id_counter()
        A = TaskA()
        B = TaskB()
        A.inputs.x = B.outputs.b
        self.failUnlessEqual(str(A.inputs), """Task1-inputs:
    x = DirectConnection: from=(Task2 prev: [] next: [1] resources: []) to=(set([Task1 prev: [2] next: [] resources: []]))
    y = DirectConnection: from=(Task0 prev: [] next: [] resources: []) to=(set([Task1 prev: [2] next: [] resources: []]))
Mode:  inputs
Owner: Task1 prev: [2] next: [] resources: []""")
        self.failUnlessEqual(str(A.outputs),  'Task1-outputs:\n    z = DirectConnection: from=(Task1 prev: [2] next: [] resources: []) to=(set([]))\nMode:  outputs\nOwner: Task1 prev: [2] next: [] resources: []')
        self.failUnlessEqual(repr(A.inputs),"""InputConnections(x = DirectConnection: from=(Task2 prev: [] next: [1] resources: []) to=(set([Task1 prev: [2] next: [] resources: []])), y = DirectConnection: from=(Task0 prev: [] next: [] resources: []) to=(set([Task1 prev: [2] next: [] resources: []])))""")

    def test4(self):
        pyutilib.workflow.reset_id_counter()
        A = TaskA()
        A.inputs['x'] = 2
        self.failUnlessEqual(A.inputs['x'].get_value(), 2)
        self.failUnlessEqual(repr(A), "Task1 prev: [] next: [] resources: []")

    def test5(self):
        pyutilib.workflow.reset_id_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        C = TaskC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A)
        w.set_options(['--i=3','--a=2'])
        try:
            w()
            self.fail("Expected TypeError because the inputs are not defined")
        except TypeError:
            pass

    def test5a(self):
        pyutilib.workflow.reset_id_counter()
        # Define tasks
        A = TaskA()
        B = TaskBB()
        C = TaskCC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        w.add(A)
        w.set_options(['--i=3','--a=2'])
        self.failUnlessEqual(w(), {'z':334})
        # This doesn't work, since we can't merge options that aren't
        # in groups.  The soln is to move to argparse, but I'll save that for later
        #w.print_help()

    def test_error1(self):
        pyutilib.workflow.reset_id_counter()
        A = TaskA()
        try:
            A.inputs['inputs'] = 2
            self.fail("Expected ValueError because we're using the reserved 'inputs' attribute name")
        except ValueError:
            pass

    def test_error2(self):
        pyutilib.workflow.reset_id_counter()
        A = TaskA()
        try:
            print A.inputs._foo
            self.fail("Should have generated error for unknown attribute _foo")
        except AttributeError:
            pass

    def test_error3(self):
        pyutilib.workflow.reset_id_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        D = TaskD()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = D.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        try:
            w.add(A)
            self.fail("Expected ValueError because there are multiple outputs with the same name")
        except ValueError:
            pass

    def test_error4(self):
        pyutilib.workflow.reset_id_counter()
        # Define tasks
        A = TaskA()
        B = TaskB()
        C = TaskC()
        # Establish connections
        A.inputs.x = B.outputs.b
        A.inputs.y = C.outputs.o
        # Create workflow
        w = pyutilib.workflow.Workflow()
        try:
            w(i=3, xx=2)
            self.fail("Expected ValueError because input 'xx' is unknown")
        except ValueError:
            pass

if __name__ == "__main__":
    unittest.main()
